Package: R4RNA
Type: Package
Title: An R package for RNA visualization and analysis
Version: 1.38.0
Date: 2015-11-02
Author: Daniel Lai, Irmtraud Meyer <irmtraud.meyer@cantab.net>
Maintainer: Daniel Lai <jujubix@cs.ubc.ca>
Depends: R (>= 3.2.0), Biostrings (>= 2.38.0)
Description: A package for RNA basepair analysis, including the visualization of basepairs as arc diagrams for easy comparison and annotation of sequence
                 and structure.  Arc diagrams can additionally be projected onto multiple sequence alignments to assess basepair conservation and
                 covariation, with numerical methods for computing statistics for each.
License: GPL-3
biocViews: Alignment, MultipleSequenceAlignment, Preprocessing,
        Visualization, DataImport, DataRepresentation,
        MultipleComparison
URL: http://www.e-rna.org/r-chie/
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/R4RNA
git_branch: RELEASE_3_22
git_last_commit: 90fdbbd
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 05:54:16 UTC; biocbuild
